#encoding: UTF-8

=begin
Pile is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

Pile is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with Pile; if not, write to the Free Software Foundation,
 Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

Copyright 2014 Detlef Reichl  detlef!reichl()gmx!org
=end

# Rgw::Pile is a container, that allows to stack a widget (the pile child)
# over an other one (the child). The pile child can be placed freely over the
# child. The pile child doesn't cover the complete child, so it will receive
# all events for the none covered areas.
#
# IMPORTANT
# =========
# Rgw::Pile is not a real Gtk::Container implementation, because
# afaik building real subclasses of Gtk::Container is for now not possible with
# Ruby-Gtk. So this is a little bit of a hack and depending of the usage it can
# produce unexpected results. So don't see this as an ready-to-use widget but
# more as a base for your own implementations.

module Rgw
    class Pile < Gtk::Box
        # Initialize a new Rgw::Pile object
        def initialize
            super(:vertical)
            @child1 = nil
            @child2 = nil
            @x = 0
            @y = 0
        end

        # Adds the bottom child to the widget.
        #
        # @param child [Gtk::Widget] The widget to add.
        def set_child child
            raise RuntimeError, 'Rgw::Pile can only contain one child' unless @child1.nil?
            @child1 = child
            self.pack_start @child1, :expand => true, :fill => true
            @child1.signal_connect_after(:size_allocate) {|widget, allo| on_allocate1 widget, allo}
        end
        
        # Adds the stacked child to the widget.
        #
        # @param child [Gtk::Widget] The widget to add. It has to be added _after_ the child.
        # @param x [Integer] Horizontal position of the child.
        # @param y [Integer] Vertical position of the child.
        #        Both positions are in pixels. If the value is positive it goes from the left
        #        respective top border of the container, if it is negative it goes from the
        #        right respective bottom border.
        def set_pile_child child, x=0, y=0
            raise RuntimeError, 'child has to be set before pile_child' if @child1.nil?
            raise RuntimeError, 'Rgw::Pile can only contain one pile_child' unless @child2.nil?
            raise ArgumentError, "unsuported class %s, expect Gtk::Widget for child" % child.class.to_s unless child.is_a? Gtk::Widget
            raise ArgumentError, "unsuported class %s, expect Integer for x" % x.class.to_s unless x.is_a? Integer
            raise ArgumentError, "unsuported class %s, expect Integer for y" % y.class.to_s unless y.is_a? Integer

            @pileBox = Gtk::Box.new :horizontal
            self.pack_start @pileBox, :expand => false, :fill => false

            
            @child2 = child
            @x = x
            @y = y
            
            @child2.has_window = false
            @pileBox.pack_start @child2, :expand => false, :fill => false
            @child2.set_parent_window self.window
            @pileBox.signal_connect_after(:size_allocate) {|widget, allo| on_allocate2 widget, allo}
        end


private
        # Expands the child, so that it also consumes the place the pile child used.
        def on_allocate1 widget, allocation
            allocation.height = self.allocation.height
            widget.size_allocate allocation
        end


        # Moves the pile child to it's destination.
        def on_allocate2 widget, allocation
            sAllocation = self.allocation

            if @x >= 0
                allocation.x = @x
            elsif @x < 0
                allocation.x = sAllocation.width - @child2.allocation.width - -@x
            end
            
            if @y >= 0
                allocation.y = @y
            elsif @y < 0
                allocation.y = sAllocation.height - @child2.allocation.height - -@y
            end
            
            widget.size_allocate allocation
        end       
    end
end
