#encoding: UTF-8

=begin
RadioRevealer is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

RadioRevealer is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with RadioRevealer; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

Copyright 2014 Detlef Reichl  detlef!reichl()gmx!org
=end

module Rgw
    class RadioRevealer < Gtk::Box
        self.type_register
        self.signal_new :revealed, GLib::Signal::RUN_FIRST, nil, GLib::Type['void'], Integer
        def signal_do_revealed foo; end

        def initialize
            super(:orientation => :vertical)

            @radioGroup = nil
            @revealers = []
            @revealerButtons = []
            @lastRevealed = nil
        end


        def create_entry head, child=nil
            button = nil
            label = nil
            if head.is_a? Gtk::Widget
                label = head
            else
                label = create_group_label head
            end

            if @radioGroup.nil?
                button = Gtk::RadioButton.new
                @radioGroup = button
            else
                button = Gtk::RadioButton.new @radioGroup
            end
            @revealerButtons << button
            button.add label

            pack_start button, :expand => false, :fill => false, :padding => 6
            button.signal_connect(:toggled) do |btn|
                @revealerButtons.each_with_index do |widget, idx|
                    if widget.active?
                        if idx != @lastRevealed
                            @revealers[@lastRevealed].reveal_child = false unless @lastRevealed.nil?
                            @revealers[idx].reveal_child = true
                            @lastRevealed = idx
                            signal_emit :revealed, idx
                        end
                        break
                    end
                end
            end
            rev = Gtk::Revealer.new
            @revealers << rev
            pack_start rev, :expand => false, :fill => false, :padding => 6

            rev.add child unless child.nil?

            if @revealers.length() == 1
                rev.reveal_child = true
                @lastRevealed = 0
            end
        end


        def set_child idx, child
            raise ArgumentError, "invalid type %s for index, expect Integer" % idx.class.to_s unless idx.is_a? Integer
            raise ArgumentError, "invalid index %i" % idx.to_i unless (0..@revealers.length() -1).cover?(idx)
            raise ArgumentError, "invalid type %s for child, expect Gtk::Widget" % child.class.to_s unless child.is_a? Gtk::Widget
            @revealers[idx].remove @revealers[idx].child unless @revealers[idx].child.nil?
            @revealers[idx].add child
        end


        def set_activate idx
            raise ArgumentError, "invalid type %s for index" % idx.class.to_s unless idx.is_a? Integer
            raise ArgumentError, "invalid index %i" % idx.to_i unless (0..@revealers.length() -1).cover?(idx)
            @revealerButtons[idx].active = true
        end
        alias_method :active=, :set_activate


        def active
            @lastRevealed
        end


        def create_group_label text
            label = Gtk::Label.new.set_xalign(0.04)
            label.markup = '<span size="large" weight="bold">' + text + '</span>'
            label
        end
    end
end
