# encoding: UTF-8

=begin
SingleBox is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

SingleBox is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with SingleBox; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

Copyright 2014 Detlef Reichl  detlef!reichl()gmx!org
=end


# prepare the Gtk::Widget class that it can contain the single box id
class Gtk::Widget
    attr_accessor :rgw_single_box_id
end


module Rgw
    class SingleBox < Gtk::Box
        def initialize dir
            super dir
            
            # only show the first child on first show
            show_id = signal_connect_after(:map) do
                signal_handler_disconnect show_id
                self.children[1..-1].each {|child| child.hide}
            end
        end


        # The user should not be able to add already shown widgets.
        def pack_start widget, *args
            widget.hide
            super
        end

        
        def pack_end widget, *args
            widget.hide
            super
        end


        def add widget
            print "Warning: Do not use Container::add on a Rgw::SingleBox; Use pack_start instead\n"
        end


        def show_by_id id
            cd = self.children
            found = false
            cd.each do |child|
                raise RuntimeError, 'SingleBox contains children without id' if child.rgw_single_box_id.nil?
                found = true if id == child.rgw_single_box_id
            end
            raise RuntimeError, 'invalid id' unless found
            cd.each do |child|
                unless id == child.rgw_single_box_id
                    child.hide
                else
                    child.show
                end
            end
        end
        
        
        def show_by_index i
            cd = self.children
            raise ArgumentError, 'invalid child index' if i < 0 or i >= cd.length
            ch.each_with_index do |child, idx|
                unless idx == i
                    child.hide
                else
                    child.show
                end
            end
        end
    end
end
