# encoding: UTF-8

=begin
SyncedSpinButton is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

SyncedSpinButton is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with SyncedSpinButton; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

Copyright 2014 Detlef Reichl  detlef!reichl()gmx!org
=end

module Rgw
    PIXMAP_PATH = File.expand_path(File.dirname(__FILE__) + '/../data/pixmaps/') + '/'
    class SyncedSpinButton < Gtk::Box
        self.type_register
        self.signal_new :value1_changed, GLib::Signal::RUN_FIRST, nil, GLib::Type['void'], Object
        self.signal_new :value2_changed, GLib::Signal::RUN_FIRST, nil, GLib::Type['void'], Object
        def signal_do_value1_changed foo; end
        def signal_do_value2_changed foo; end


        def initialize min, max, step, locked
            super(:orientation => :horizontal)
            @locked = locked

            @s1 = Gtk::SpinButton.new min, max, step
            @s2 = Gtk::SpinButton.new min, max, step

            @s1.signal_connect(:value_changed) do
                signal_emit :value1_changed, @s1.value
                next unless @locked
                @s2.value = @s1.value if @s2.value != @s1.value
            end

            @s2.signal_connect(:value_changed) do
                signal_emit :value2_changed, @s2.value
                next unless @locked
                @s1.value = @s2.value if @s1.value != @s2.value
            end

            lb = Gtk::ToggleButton.new
            lb.active = @locked
            img = Gtk::Image.new(:file => Rgw::PIXMAP_PATH + (locked ? 'lock16.png' : 'unlock16.png'))
            lb << img
            lb.signal_connect(:toggled) do
                @locked = lb.active?
                img.file = @locked ? Rgw::PIXMAP_PATH + 'lock16.png' : PIXMAP_PATH + 'unlock16.png'
                @s2.value = @s1.value if @locked
            end

            pack_start @s1, :expand => true, :fill => true, :padding => 0
            pack_start lb, :expand => false, :fill => false, :padding => 0
            pack_start @s2, :expand => true, :fill => true, :padding => 0
        end


        def set_values v1, v2
            raise RangeError, 'value 1 out of range' if v1 < @s1.range.min or v1 > @s1.range.max
            raise RangeError, 'value 2 out of range' if v2 < @s2.range.min or v1 > @s2.range.max
            @s1.value = v1
            @s2.value = v2
        end


        def get_values
            [@s1.value, @s2.value]
        end
    end
end
