#encoding: UTF-8

=begin
Types is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

Types is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with Types; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

Copyright 2014 Detlef Reichl  detlef!reichl()gmx!org
=end


# this are some extension to standard types

class Float
    def near f
        self * 1.000000001 > f and self * 0.999999999 < f
    end
end


class Numeric
    def min a
        self <= a ? self : a
    end


    def max a
        self >= a ? self : a
    end


    def in a, b
        a.min(b) < self && self < a.max(b)
    end


    def clamp min, max
        s = self.max(min)
        s.min(max)
    end
end


class Fifo < Array
    def initialize l
        raise ArgumentError, "length has to be >= 1" if l.to_i < 1
        @lng = l.to_i
        super()
    end

    def push item
        super
        self.delete_at(0) if self.length > @lng
    end

    def max
        @lng
    end

    def resize l
        @lng = l
        while(self.length > @lng)
            self.delete_at(0)
        end
    end

    alias << push
end


class Rectangle
    def initialize x=0.0, y=0.0, w=0.0, h=0.0
        @x = x
        @y = y
        @width = w
        @height = h
    end
    attr_reader :x, :y, :width, :height

    def x=(x)
        raise ArgumentError, 'invallid type for pos x' unless x.is_a? Numeric
        @x = x.to_f
    end


    def y=(y)
        raise ArgumentError, 'invallid type for pos y' unless y.is_a? Numeric
        @y = y.to_f
    end


    def width=(w)
        raise ArgumentError, 'invallid type for width' unless w.is_a? Numeric
        raise ArgumentError, 'width has to be >= 0' unless w >= 0.0
        @width = w.to_f
    end


    def height=(h)
        raise ArgumentError, 'invallid type for height' unless h.is_a? Numeric
        raise ArgumentError, 'height has to be >= 0' unless h >= 0.0
        @height = h.to_f
    end


    def in x, y
        x >= @x and x <= @x + @width and y >= @y and y <= @y + @height
    end


    def grow!(val)
        @x -= val
        @y -= val
        @width += (val * 2.0)
        @height += (val * 2.0)
    end
end


